#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include "NcTask.h"

#ifndef ASSERT
#ifdef DEBUG
extern void __error__(char *pcFilename, uint32_t ui32Line);
#define ASSERT(expr) do                                                       \
                     {                                                        \
                         if(!(expr))                                          \
                         {                                                    \
                             __error__(__FILE__, __LINE__);                   \
                         }                                                    \
                     }                                                        \
                     while(0)
#else
#define ASSERT(expr)
#endif//DEBUG
#endif//ASSERT

Task * TaskGet(uint8_t index)
{
	ASSERT(index < g_numberOfTasks);
	return &g_tasks[ index ];
}

bool TaskInitAll(void* p)
{
	int i;
	for(i=0; i<g_numberOfTasks; i++)
	{
		Task * t = TaskGet(i);
		if (!t->pfnInit(p))
			return false;
	}
	return true;
}

void TaskRunAll(void* p)
{
	int i;
	for(i=0; i<g_numberOfTasks; i++)
	{
		Task * t = TaskGet(i);
		if (t->pfnRun!=NULL)
		{
			pthread_create( &t->thread, NULL, t->pfnRun, p );
		}
		else
			t->thread = 0;
	}
}

void TaskShutdownAll()
{
	int i;
	for(i=0; i<g_numberOfTasks; i++)
	{
		Task * t = TaskGet(i);
		if (t->pfnShutdown())
			continue;
		pthread_cancel( t->thread );
		t->thread = 0;
	}
	for(i=0; i<g_numberOfTasks; i++)
	{
		Task * t = TaskGet(i);
		if (t->thread!=0)
			pthread_join( t->thread, NULL );
	}
}
