#ifndef CONFIGURATION_H_
#define CONFIGURATION_H_

#include <stdint.h>
#include "ConfigEssentials.h"
#include "NcClosers.h"
#include "Serializer.h"
#include "NcString.h"

#if !_WIN32 && !_WIN64
#include <termios.h>
#endif

#define CONFIG_UART "uart"
#define CONFIG_SPEED "speed"
#define CONFIG_RECVPORT "recvPort"
#define CONFIG_SENDPORT "sendPort"
#define CONFIG_SERIALIZER "serializer"
#define CONFIG_CTSTIMEOUTSECS "ctsTimeoutSecs"
#define CONFIG_CTSSOURCE "ctsSource"
#define CONFIG_GPIOPIN "gpioPin"
#define CONFIG_DELIMITER "delimiter"

#define CONFIG_SPEED_57600 "B57600"
#define CONFIG_SPEED_115200 "B115200"
#define CONFIG_SPEED_230400 "B230400"

#define CONFIG_CTSSOURCE_IOCTL "ioctl"
#define CONFIG_CTSSOURCE_GPIO "gpio"

class tUartConfig
{
public:
	virtual ~tUartConfig() {}
	virtual const char * getUart() = 0;
	virtual void setUart(const char * v) = 0;
	virtual speed_t getSpeed() = 0;
	virtual void setSpeed(speed_t v) = 0;
	virtual const char * getCtsSource() = 0;
	virtual void setCtsSource(const char * v) = 0;
};

class tRecvConfig
	: public virtual tUartConfig
	, public virtual ConfigEssentials
{
public:
	virtual ~tRecvConfig() {}
	virtual uint16_t getRecvPort() = 0;
	virtual void setRecvPort(uint16_t v) = 0;
};

class tSendConfig
	: public virtual tUartConfig
	, public virtual ConfigEssentials
{
public:
	virtual ~tSendConfig() {}
	virtual uint16_t getSendPort() = 0;
	virtual void setSendPort(uint16_t v) = 0;
	virtual uint16_t getCtsTimeoutSecs() = 0;
	virtual void setCtsTimeoutSecs(uint16_t v) = 0;
};

class tGpioConfig
{
public:
	virtual ~tGpioConfig() {}
	virtual int getGpioPin() = 0;
	virtual void setGpioPin(int v) = 0;
};

typedef class tConfiguration
	: public virtual tUartConfig
	, public virtual tRecvConfig
	, public virtual tSendConfig
	, public virtual tGpioConfig
{
	const char * _uart;
	speed_t _speed;
	uint16_t _recvPort;
	uint16_t _sendPort;
	uint16_t _ctsTimeoutSecs;
	const char * _ctsSource;
	int _pin;
	const char * _delimiter;
	char * _delimiterFormatted;

public:
	virtual const char * getUart() { return _uart; }
	virtual void setUart(const char * v) { _uart = v; }
	virtual uint16_t getRecvPort() { return _recvPort; }
	virtual void setRecvPort(uint16_t v) { _recvPort = v; }
	virtual uint16_t getSendPort() { return _sendPort; }
	virtual void setSendPort(uint16_t v) { _sendPort = v; }
	virtual speed_t getSpeed() { return _speed; }
	virtual void setSpeed(speed_t v) { _speed = v; }
	virtual uint16_t getCtsTimeoutSecs() { return _ctsTimeoutSecs; }
	virtual void setCtsTimeoutSecs(uint16_t v) { _ctsTimeoutSecs = v; }
	virtual const char * getCtsSource() { return _ctsSource; }
	virtual void setCtsSource(const char * v) { _ctsSource = v; }
	virtual int getGpioPin() { return _pin; }
	virtual void setGpioPin(int v) { _pin = v; }
	virtual const char * getDelimiter() { return _delimiter; }
	virtual void setDelimiter(const char * v)
	{
		_delimiter = v;
		Free((void**)&_delimiterFormatted);
		_delimiterFormatted = StringUnEscape(v);
	}
	virtual const char * getDelimiterFormatted() { return _delimiterFormatted; }
	tConfiguration()
	{
		_uart = 0;
		_speed = 0;
		_recvPort = 0;
		_sendPort = 0;
		_ctsTimeoutSecs = 0;
		_ctsSource = 0;
		_pin = 0;
		_delimiter = 0;
		_delimiterFormatted = 0;
	}
} Configuration;

typedef enum tTaskRunEnum
{
	Stopped,
	Started,
	ReLoadConfig
} TaskRunEnum;

bool GetConfig(int argc, char** args, Configuration * c);
bool GetConfigFromFile(char * file, Configuration * c);
void GetConfigDefault(Configuration * c);
const char * GetDefaultUart();
const char * MapSpeed(int speed);
void PrintConfig(Configuration * c);

#endif /* CONFIGURATION_H_ */
