#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <stdarg.h>
#include <stddef.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "Configuration.h"
#include "Constants.h"
#include "CtsTask.h"
#include "NcApi.h"
#include "NcPrintErr.h"
#include "RecvTask.h"
#include "SendTask.h"
#include "Serializer.h"
#include "NcTask.h"

#define SIZEOF(x) ((sizeof(x)/sizeof(0[x])) / ((uint32_t)(!(sizeof(x) % sizeof(0[x])))))

Task g_tasks[] =
{
	 {RecvTaskInit, RecvTaskRun, RecvTaskShutdown}
	,{SendTaskInit, SendTaskRun, SendTaskShutdown}
	,{CtsTaskInit, CtsTaskRun, CtsTaskShutdown}
};
uint8_t g_numberOfTasks = SIZEOF(g_tasks);

Configuration g_config;
ConfigEssentials g_config_essentials;
int g_argc;
char** g_argv;

tNcApiRxHandlers g_ncRx;
tNcApi g_ncApi[1];
uint8_t g_numberOfNcApis = 1;



int Usage(const char* level)
{
	Configuration c;
	GetConfigDefault(&c);
	printf("%s: %s\n", level, g_error);
	printf("NeoCortecGateway [-c config-file]\n");
	printf("NeoCortecGateway [-C key1=value1[,keyN=valueN...]]\n");
	printf("Default UART: %s\n", c.getUart());
	printf("Default Speed: %s\n", MapSpeed(c.getSpeed()));
	printf("Default Cts: %s\n", c.getCtsSource());
	printf("Default RecvPort: %d\n", c.getRecvPort());
	printf("Default SendPort: %d\n", c.getSendPort());
	printf("Default Serializer: %s\n", c.getSerializerName());
	return 1;
}



volatile int DoLoop;

void sigint_handler(int sig)
{
	DoLoop=0;
}


void WaitForUser()
{
    signal(SIGINT, sigint_handler);
    signal(SIGTERM, sigint_handler);
	
    

	DoLoop=1;
	while (DoLoop)
	{
		sleep(-1);
		
	}

}

bool DoInit(Configuration * c, ConfigEssentials * c_essentials);

void SignalHandler(int sig)
{
	printf("Caught signal %d: reloading config\n", sig);
	TaskShutdownAll();
	Configuration * c = &g_config;
	ConfigEssentials * c_essentials = &g_config_essentials;
	if (!GetConfig(g_argc, g_argv, c) || !DoInit(c,c_essentials))
	{
		printf("ERROR: Failed initializing after reloading of config:\n%s\n", g_error);
		exit(EXIT_FAILURE);
	}
	PrintConfig(c);
	TaskRunAll(c);
}

bool SetSignalHandler()
{
	if (signal(SIGUSR1, SignalHandler)==SIG_ERR)
		return PrintErr("Failed to set signal handler for SIGUSR1. errno=%s", PrintErrNo(errno));
	return true;
}

int DoMain(Configuration * c)
{
	PrintConfig(c);
	TaskRunAll(c);
	WaitForUser();
	TaskShutdownAll();
	return (EXIT_SUCCESS);
}

bool DoInit(Configuration * c, ConfigEssentials * c_essentials)
{
	{

		tNcApiRxHandlers * h = &g_ncRx;
		memset( h, 0, sizeof(tNcApiRxHandlers));

		h->pfnReadCallback = UartReadCallbackSpecialized;

		h->pfnHostAckCallback = UartHostAckCallback;
		h->pfnHostNAckCallback = UartHostNAckCallback;
		h->pfnHostDataCallback = UartHostDataCallback;
		h->pfnHostDataHapaCallback = UartHostDataHapaCallback;
		h->pfnWesStatusCallback = UartWesStatusCallback;
		h->pfnWesSetupRequestCallback = UartWesSetupRequestCallback;
		
		h->pfnHostUappDataCallback = UartHostUappDataCallback;
		h->pfnHostUappDataHapaCallback = UartHostUappDataHapaCallback;
		h->pfnNodeInfoReplyCallback = UartNodeInfoReplyCallback;
		h->pfnNeighborListReplyCallback = UartNeighborListReplyCallback;
		h->pfnNetCmdResponseCallback = UartNetCmdReplyCallback;
		
	
		NcApiInit();
	
		g_ncApi[0].NcApiRxHandlers=&g_ncRx;
	}
	return TaskInitAll(c)
        && SetSignalHandler();
}

int main(int argc, char** argv)
{
	setbuf(stdout, NULL);
	g_argc = argc;
	g_argv = argv;
	Configuration * c = &g_config;
	ConfigEssentials * c_essentials = &g_config_essentials;
	printf("NeoCortecGateway " __DATE__ " " __TIME__ "\n");
	if (GetConfig(g_argc, g_argv, c)
	 && DoInit(c, c_essentials))
		return DoMain(c);
	return Usage("ERROR");
}
